#
# Copyright (c) 2008-2018 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

import math
from itertools import tee

from zope.schema.fieldproperty import FieldProperty

from pyams_content import _
from pyams_content.features.search import ISearchFolder
from pyams_content.features.search.portlet.interfaces import ISearchResultsPortletSettings
from pyams_content.shared.view.interfaces import RELEVANCE_ORDER, VISIBLE_PUBLICATION_DATE_ORDER
from pyams_portal.portlet import Portlet, PortletSettings, portlet_config
from pyams_utils.factory import factory_config
from pyams_utils.interfaces import VIEW_PERMISSION
from pyams_utils.request import check_request
from pyams_utils.traversing import get_parent


SEARCH_RESULTS_PORTLET_NAME = 'pyams_content.portlet.search.results'


@factory_config(provided=ISearchResultsPortletSettings)
class SearchResultsPortletSettings(PortletSettings):
    """Search results portlet settings"""

    title = FieldProperty(ISearchResultsPortletSettings['title'])
    allow_empty_query = FieldProperty(ISearchResultsPortletSettings['allow_empty_query'])
    force_canonical_url = FieldProperty(ISearchResultsPortletSettings['force_canonical_url'])

    @staticmethod
    def has_user_query(request):
        params = request.params
        return params.get('user_search', '').strip()

    @staticmethod
    def _get_items(request=None, start=0, length=10, limit=None, ignore_cache=False):
        context = get_parent(request.context, ISearchFolder)
        if context is None:
            raise StopIteration
        else:
            if request is None:
                request = check_request()
            params = request.params
            order_by = params.get('order_by', context.order_by)
            if (order_by == RELEVANCE_ORDER) and \
                    not SearchResultsPortletSettings.has_user_query(request):
                request.GET['order_by'] = order_by = VISIBLE_PUBLICATION_DATE_ORDER
            yield from context.get_results(context, order_by,
                                           reverse=order_by != RELEVANCE_ORDER,
                                           limit=limit,
                                           start=int(start),
                                           length=int(length),
                                           ignore_cache=ignore_cache,
                                           get_count=True,
                                           request=request)

    def get_items(self, request=None, start=0, length=10, limit=None, ignore_cache=False):
        if not (self.allow_empty_query or self.has_user_query(request)):
            yield from iter(((), 0), )
        else:
            check, items = tee(
                SearchResultsPortletSettings._get_items(request, start, length, limit,
                                                        ignore_cache))
            try:
                next(check)
            except StopIteration:
                yield from iter(((), 0), )
            else:
                yield from items

    @staticmethod
    def get_pages(start, length, count):
        start = int(start) + 1
        length = int(length)
        current = math.ceil(start / length)
        nb_pages = math.ceil(count / length)
        return current, nb_pages


@portlet_config(permission=VIEW_PERMISSION)
class SearchResultsPortlet(Portlet):
    """Search results portlet"""

    name = SEARCH_RESULTS_PORTLET_NAME
    label = _("Search results")

    toolbar_css_class = 'fa fa-fw fa-2x fa-search-plus'

    settings_factory = ISearchResultsPortletSettings
